/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Paste and Go 2.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Holio Lin <gaghiel@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var MenuRoll = {
    menuRolling: false,
    currentItemIndex: 0,
    searchbar: null,
    popup: null,
    //engines: null, // dynamic
    items: null,
    rollKey: null,
    delayMS: 0,
    // searchService: null,
    rdfService: null, // for 1.5 use
    internetSearchDS: null, // for 1.5 use

    startup: function(searchbar, rollKey, delayMS) {
        MenuRoll.searchbar = searchbar;

        // for version compatible
        if ('_popup' in searchbar) { // if 2.0+
            MenuRoll.popup = searchbar._popup;
            MenuRoll.items = MenuRoll.popup.childNodes;
        } else { // if 1.5
            MenuRoll.popup = document.getAnonymousElementByAttribute(searchbar,
                    'anonid', 'searchbar-popup');
            var nodes = MenuRoll.popup.childNodes;

            // in 1.5, popup.childNodes[0] here is a 'xul:template', so skip
            MenuRoll.items = [];
            var n = nodes.length;
            for (var i = 1; i < n; i++)
                MenuRoll.items.push(nodes[i]);
        }

        //MenuRoll.engines = searchbar._engines;
        MenuRoll.rollKey = rollKey;

        window.addEventListener("keypress", MenuRoll.onceKeyPress, true);
        window.addEventListener("keyup", MenuRoll.onceKeyUp, true);

        MenuRoll.popup.addEventListener("popuphidden", MenuRoll.onceMenuHidden, false);

        MenuRoll.popup.addEventListener("DOMMenuItemActive", MenuRoll.ItemActive, true);
        MenuRoll.popup.addEventListener("DOMMenuItemInactive", MenuRoll.ItemInactive, true);

        MenuRoll.delayMS = delayMS;
    },

    shutdown: function() {
        window.removeEventListener("keypress", MenuRoll.onceKeyPress, true);
        window.removeEventListener("keyup", MenuRoll.onceKeyUp, true);
    },

    getEngines: function() { // dynamic
        // for version compatible
        if ('_engines' in MenuRoll.searchbar) // 2.0 - 3.0
            return MenuRoll.searchbar._engines;
        else { // searchbar of 1.5 have no field '_engines'
            if (!this.rdfService) { // lazy initial
                this.rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                      .getService(Components.interfaces.nsIRDFService);
                this.internetSearchDS = this.rdfService.GetDataSource("rdf:internetsearch");
            }

            var engines = [];

            const kNC_Root  = this.rdfService.GetResource("NC:SearchEngineRoot");
            const kNC_Child = this.rdfService.GetResource("http://home.netscape.com/NC-rdf#child");
            const kNC_Name  = this.rdfService.GetResource("http://home.netscape.com/NC-rdf#Name");

            var arcs = this.internetSearchDS.GetTargets(kNC_Root, kNC_Child, true);
            while (arcs.hasMoreElements()) {
                var rEngine = arcs.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
                engines.push(rEngine.Value);
            }

            return engines;
            // clone code from mozilla-1.5/browser/base/content/search.xml
        }
    },

    ItemActive: function(event) {
        var currentItem = MenuRoll.items[MenuRoll.currentItemIndex];
        if (MenuRoll.menuRolling && event.target != currentItem)
            event.target.setAttribute("_moz-menuactive", "false"); // anti default interaction
    },

    ItemInactive: function(event) {
        var currentItem = MenuRoll.items[MenuRoll.currentItemIndex];
        if (MenuRoll.menuRolling && event.target == currentItem)
            event.target.setAttribute("_moz-menuactive", "true"); // anti default interaction
    },

    onceKeyPress: function(event) {
        // bug: accel key here did't ref to pref "extensions.pasteandgo2.GoAccelKeys"
        if ((String.fromCharCode(event.charCode).toUpperCase() == MenuRoll.rollKey.toUpperCase()) 
                && event.ctrlKey && event.shiftKey && !event.altKey) { 

            if (MenuRoll.menuRolling) {
                MenuRoll.items[MenuRoll.currentItemIndex].setAttribute("_moz-menuactive", "false");
                MenuRoll.currentItemIndex = (MenuRoll.currentItemIndex + 1) % MenuRoll.getEngines().length; // step
            } else {
                MenuRoll.menuRolling = true;
                
                if (MenuRoll.delayMS == 0)
                    MenuRoll.popup.click();
                else {
                    window.setTimeout( function() {
                        if (MenuRoll.menuRolling) {  // synchronization
                            var keep = PrefsObserver.pgSearchTextbox.currentEngine;
                            MenuRoll.popup.click(); // show
                            // fixedbug: under 1.5 click() of popup will clean currentEngine
                            PrefsObserver.pgSearchTextbox.currentEngine = keep;
                            // currentEngine's setter handles the rest
                        }
                    }, MenuRoll.delayMS);
                }
            }
            MenuRoll.items[MenuRoll.currentItemIndex].setAttribute("_moz-menuactive", "true");

            event.stopPropagation();
            event.preventDefault();
        }
    },

    onceMenuHidden: function(event) { // reset state
        MenuRoll.items[MenuRoll.currentItemIndex].setAttribute("_moz-menuactive", "false");
        MenuRoll.currentItemIndex = 0;
        MenuRoll.menuRolling = false;
    },

    onceKeyUp: function(event) {
        if (MenuRoll.menuRolling && event.keyCode == event.DOM_VK_CONTROL) {// && !event.altKey && !event.metaKey)) {
            MenuRoll.menuRolling = false; // avoid synchronization failedd
            
            searchByEngine(MenuRoll.getEngines()[MenuRoll.currentItemIndex]);

            MenuRoll.popup.hidePopup();   // won't invoke onceMenuHidden
            MenuRoll.onceMenuHidden();
        }
    },

} // end of MenuRoll
